==> OpenCL-0.8.3.tgz: OpenCL-0.8.3/src/mp_misc.cpp <==
/*************************************************
* Misc MP Functions Source File                  *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/mp_core.h>

extern "C" {

/*************************************************
* Division Core                                  *
*************************************************/
bool bigint_divcore(u32bit q, u32bit y_1, u32bit y_2,
                    u32bit x1, u32bit x2, u32bit x3)
   {
   u32bit last = 0;
   u64bit product = 0;

   product = (u64bit)q * y_2;
   y_2 = (product & 0xFFFFFFFF);

   product = (u64bit)q * y_1 + ((product >> 32) & 0xFFFFFFFF);
   y_1 = (product & 0xFFFFFFFF);
   last = (product >> 32) & 0xFFFFFFFF;

   if(last > x1) return true;
   if(last < x1) return false;
   if(y_1 > x2)  return true;
   if(y_1 < x2)  return false;
   if(y_2 > x3)  return true;
   if(y_2 < x3)  return false;
   return false;
   }

/*************************************************
* Compare                                        *
*************************************************/
s32bit bigint_cmp(const u32bit* x, u32bit x_size,
                  const u32bit* y, u32bit y_size)
   {
   if(x_size < y_size) { return (-bigint_cmp(y, y_size, x, x_size)); }

   while(x_size > y_size)
      {
      if(x[x_size-1])
         return 1;
      x_size--;
      }
   for(u32bit j = x_size; j > 0; j--)
      {
      if(x[j-1] > y[j-1]) return 1;
      if(x[j-1] < y[j-1]) return -1;
      }
   return 0;
   }

}

==> OpenCL-0.8.4.tgz: OpenCL-0.8.4/src/mp_misc.cpp <==
/*************************************************
* Misc MP Functions Source File                  *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/mp_core.h>

extern "C" {

/*************************************************
* Division Core                                  *
*************************************************/
bool bigint_divcore(u32bit q, u32bit y_1, u32bit y_2,
                    u32bit x1, u32bit x2, u32bit x3)
   {
   u64bit product = (u64bit)q * y_2;
   y_2 = (product & 0xFFFFFFFF);

   product = (u64bit)q * y_1 + ((product >> 32) & 0xFFFFFFFF);
   y_1 = (product & 0xFFFFFFFF);
   u32bit last = (product >> 32) & 0xFFFFFFFF;

   if(last > x1) return true;
   if(last < x1) return false;
   if(y_1 > x2)  return true;
   if(y_1 < x2)  return false;
   if(y_2 > x3)  return true;
   if(y_2 < x3)  return false;
   return false;
   }

/*************************************************
* Compare                                        *
*************************************************/
s32bit bigint_cmp(const u32bit* x, u32bit x_size,
                  const u32bit* y, u32bit y_size)
   {
   if(x_size < y_size) { return (-bigint_cmp(y, y_size, x, x_size)); }

   while(x_size > y_size)
      {
      if(x[x_size-1])
         return 1;
      x_size--;
      }
   for(u32bit j = x_size; j > 0; j--)
      {
      if(x[j-1] > y[j-1]) return 1;
      if(x[j-1] < y[j-1]) return -1;
      }
   return 0;
   }

}
